import java.applet.*;
import java.awt.*;
import javax.swing.*;
import java.io.*;


public class Zini extends Applet
{
    public Modell modell;
    public Integrator integrator;

    public int anz=0;
    public double[] y;
    public double[] t;
    public double dt;

    private double[] yneu = null;
    private double[] yalt;

    private int XOFF=10;
    private int YOFF=260;
    private int HOEHE=450;
    private int BREITE=1100;

    private double XMIN=0.0,XMAX=1.0,YMIN=0.0,YMAX=1.0;

    private void anpassen()
    {
        int anz_neu = 1+(int)(modell.p("tdur")/modell.p("dt"));
        if(anz!=anz_neu)
        {
            anz = anz_neu;
            y=new double[anz];
            t=new double[anz];
        }

        //Totzeit-Array anpassen:
        //Zwischenspeicher fr das Totzeitverhalten bereitstellen:
        int tanz = (int)(modell.p("Tt")/modell.p("dt"));
        if(tanz!=modell.utot.length)
        {
            modell.utot = new double[tanz];
        }
        modell.ringindex=0;
        for(int i=0;i<modell.utot.length;i++)
            modell.utot[i]=0.0;
    }

    public void simulieren()
    {
        anpassen();
        t[0]=modell.p("t0");
        y[0]=0.0;
        dt = modell.p("dt");

        for(int i=0;i<yalt.length;i++)
            yalt[i]=0.0;

        for(int i=1;i<anz;i++)
        {
            t[i]=t[i-1]+dt;
            yneu = integrator.zeitschritt(yalt,t[i],dt);
            for(int k=0;k<yalt.length;k++)
                yalt[k]=yneu[k];
            y[i]=yneu[0];
        }
    }

    public void init()
    {
        setLayout(null);

        Farben.init();

        modell     = new Modell();
        integrator = new Integrator(modell);

        yalt = new double[modell.holeAnzahlGleichungen()];

        simulieren();

        for(int i=0;i<modell.par.length;i++)
        {
            new Schieber(this,
                         modell.param_beschreibung[i*7+1], 
                         modell.param_beschreibung[i*7+5],
                         5, 
                         modell.par,
                         i,
                         Double.parseDouble(modell.param_beschreibung[i*7+2]),
                         Double.parseDouble(modell.param_beschreibung[i*7+3]),
                         "");
        }        
    }

    public void start()
    {
    }

    private int xskal(double x)
    {
        return (int)( 0.5 + ((double)(BREITE-1)*(x-XMIN))/(XMAX-XMIN)  );
    }
    private int yskal(double y)
    {
        return (int)( 0.5 + ((double)(HOEHE-1)*(y-YMIN))/(YMAX-YMIN)  );
    }
    
    public void paint(Graphics g)
    {
        g.setColor(Farben.SCHWARZ);
        g.drawRect(XOFF-1,YOFF-1,BREITE,HOEHE);

        XMIN = t[0];
        XMAX = t[anz-1];
        YMIN = y[0];
        for(int i=0;i<anz;i++)
            if(y[i]<YMIN)
                YMIN=y[i];
        YMAX = y[0];
        for(int i=0;i<anz;i++)
            if(y[i]>YMAX)
                YMAX=y[i];

/*
        if(YMIN>modell.p("w"))
            YMIN=modell.p("w");
        if(YMAX<modell.p("w"))
            YMAX=modell.p("w");
*/
        //w einzeichnen:
        g.setColor(Farben.GRUEN);
        g.drawLine(XOFF,YOFF+HOEHE-1-yskal(modell.p("w")),XOFF+BREITE-1,YOFF+HOEHE-1-yskal(modell.p("w")));
        
        //y einzeichnen:
        g.setColor(Farben.BLAU);
        for(int i=0;i<anz-1;i++)
        {
            g.drawLine(XOFF+xskal(t[i]),YOFF+HOEHE-1-yskal(y[i]),XOFF+xskal(t[i+1]),YOFF+HOEHE-1-yskal(y[i+1]));             
        }
        //Maxima angeben
        g.setColor(Farben.GRUEN);
        for(int i=1;i<anz-1;i++)
        {
            if(y[i]>y[i-1] && y[i]>=y[i+1])
            {
                int pp=i+1;
                while(pp<anz && y[pp]==y[i])
                    pp++;
                if(pp+1<anz && y[pp+1]<y[i])
                {
                    int ii=(i+pp)/2;
//                    g.drawString("["+String.format("%.3e",t[ii])+"|"+String.format("%.3e",y[ii])+"]",XOFF+xskal(t[ii]),YOFF+HOEHE-1-yskal(y[ii])-10);
                    g.drawString("["+String.format("%.3e",t[ii])+"|",XOFF+xskal(t[ii]),YOFF+HOEHE-1-yskal(y[ii])-15);
                    g.drawString(""+String.format("%.3e",y[ii])+"]",XOFF+xskal(t[ii]),YOFF+HOEHE-1-yskal(y[ii])-5);
                }
            }
        }
        //Grenzen angeben
        g.setColor(Farben.BLAU);
        g.drawString(""+YMAX,XOFF+BREITE,YOFF-10);
        g.drawString(""+YMIN,XOFF+BREITE,YOFF+HOEHE-10);
      
    }

    public static void main(String[] args)
    {
        Zini z = new Zini();
        z.init();
        z.start();
        z.repaint();

        JFrame mF = new JFrame("Reglerauslegung nach Ziegler Nichols");
        mF.getContentPane().add(z);
        mF.pack();
        mF.setSize(1200,760);
        mF.setLocation(0,0);
        mF.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        mF.setVisible(true);
    }
}